/*---------------------------------------------------------------------------*
 *
 * Project
 *        fXPlayMidi
 *
 * $RCSfile: bldabms.c,v $
 * $Date: 1997/09/14 01:27:56 $
 * $Author: klmcw $
 * $Revision: 1.4 $
 *
 *---------------------------------------------------------------------------*
 * $Log: bldabms.c,v $
 * Revision 1.4  1997/09/14 01:27:56  klmcw
 * Include #!fXPlayMidi -a when building albums. It actually uses argv[0].
 *
 * Revision 1.3  1997/04/26 08:28:39  klmcw
 * Added support for ".au" files.
 *
 * Revision 1.2  1996/04/19 13:13:59  klmcw
 * Needed malloc.h -- oops.
 *
 * Revision 1.1  1996/03/19 13:55:07  klmcw
 * Initial revision
 *
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
#include <dirent.h>
#include <limits.h>
#include <malloc.h>
#include <memory.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>

#include "album.h"
#include "bldabms.h"
#include "dllist/dlist.h"

/*---------------------------------------------------------------------------*/

static int  bldabmsAddToList(const char *File, DL_LIST thisList);
static int  bldabmsCmp(void *v1, void *v2);
static void bldabmsItemFree(void *keyOrData, void *info);
static int  bldabmsNeedToAddFile(struct dirent *dirEnt);
static int  bldabmsProcDir(const char *TopDir);
static int  bldabmsProcDirBldList(const char *thisDir, DL_LIST thisList);
static int  bldabmsProcDirProcList(const char *thisDir, DL_LIST thiList);

/*---------------------------------------------------------------------------*/

static DLL_SETUP bldabmsSetup = NULL;

static char* bldabmsArgv0 = NULL;

/*---------------------------------------------------------------------------*/
int
bldabms(int argc, char * const argv[], int optind)
{
  register int i;

  if((bldabmsSetup = dll_setup(bldabmsCmp, NULL)) == NULL) {
    printf("Could not create doubly-linked setup.\n");
    return(-1);
  }

  bldabmsArgv0 = argv[0];

  for(i = optind; i < argc; i++)
    if(bldabmsProcDir(argv[i]) < 0)
      break;

  dll_freeSetup(bldabmsSetup);

  return(0);
}
/*---------------------------------------------------------------------------*/
static int
bldabmsAddToList(const char *File, DL_LIST thisList)
{
  register int rc;
  char *filep = strdup(File);

  if(filep == NULL) return(-1);

  rc = dll_insert(thisList, filep, filep);
  if(rc <= 0) printf("Did Not Add '%s'\n", filep);

  return(rc);
}
/*---------------------------------------------------------------------------*/
static int
bldabmsCmp(void *v1, void *v2)
{
  char *cp1 = (char *)v1, *cp2 = (char *)v2;
  register int rc = strcmp(cp1, cp2);

  return(rc);
}
/*---------------------------------------------------------------------------*/
static void
bldabmsItemFree(void *keyOrData, void *info)
{
  if(keyOrData != NULL) free(keyOrData);
}
/*---------------------------------------------------------------------------*/
static int
bldabmsNeedToAddFile(struct dirent *dirEnt)
{
  int len = strlen(dirEnt->d_name);

  if((len >= 5) && !strncmp(dirEnt->d_name + len - 4, ".mid", 4)) return(1);
  if((len >= 4) && !strncmp(dirEnt->d_name + len - 3, ".au", 3)) return(1);

  return(0);
}
/*---------------------------------------------------------------------------*/
static int
bldabmsProcDir(const char *TopDir)
{
  register int rc = 0;
  DL_LIST thisList = NULL;

  if((thisList = dll_new(bldabmsSetup)) == NULL) {
    printf("Could not create doubly-linked list.\n");
    return(-1);
  }

  if(!(rc = bldabmsProcDirBldList(TopDir, thisList)))
    rc = bldabmsProcDirProcList(TopDir, thisList);

  /* Only supply key free function, because:
   *
   * Key and Data are the same in our list(s);
   *
   * gcc-lib free/malloc gets terribly confused when the same address
   * is free()-ed multiple times!
   */
  dll_destroy(thisList, bldabmsItemFree, NULL, NULL);

  return(rc);
}
/*---------------------------------------------------------------------------*/
static int
bldabmsProcDirBldList(const char *thisDir, DL_LIST thisList)
{
  static char Msg[_POSIX_PATH_MAX+17];
  char FileOrDir[_POSIX_PATH_MAX+1];
  register int rc = 0;
  DIR *Dir;
  struct dirent *dirEnt;
  struct stat statBuf;

  if((Dir = opendir(thisDir)) == NULL) {
    memset(Msg, '\0', sizeof(Msg));
    sprintf(Msg, "bldabmsProcDirBldList(%s)", thisDir);
    perror(Msg);

    return(-1);
  }

  for(dirEnt = readdir(Dir); !rc && (dirEnt != NULL); dirEnt = readdir(Dir)) {
    if(!dirEnt->d_ino || (*dirEnt->d_name == '.')) continue;

    memset(FileOrDir, '\0', sizeof(FileOrDir));
    sprintf(FileOrDir, "%s/%s", thisDir, dirEnt->d_name);

    if(stat(FileOrDir, &statBuf) < 0) {
      memset(Msg, '\0', sizeof(Msg));
      sprintf(Msg, "bldabmsProcDirBldList(%s) - %s", thisDir, dirEnt->d_name);
      perror(Msg);

      continue;
    }

    if(S_ISDIR(statBuf.st_mode)) {
      if(bldabmsProcDir(FileOrDir) < 0)
	rc = -1;
    } else if(bldabmsNeedToAddFile(dirEnt)) 
      if(!bldabmsAddToList(FileOrDir, thisList))
	rc = -1;
  }

  (void)closedir(Dir);
  
  return(rc);
}
/*---------------------------------------------------------------------------*/
static int
bldabmsProcDirProcList(const char *thisDir, DL_LIST thisList)
{
  static char const *ap = NULL;
  static char Msg[_POSIX_PATH_MAX+17];
  char const *cp = NULL;
  char Album[_POSIX_PATH_MAX+1];
  register int rc = 0;
  FILE *Fp = NULL;
  void *key, *node;

  for(key = dll_first(thisList, &node); key != NULL; key = dll_next(thisList, &node)) {
    if(ap == NULL)
      if((ap = albumPathFind(NULL)) == NULL) {
	printf("Could not find Album Path...\n");
	return(-1);
      }

    if(cp == NULL)
      if((cp = strrchr(thisDir, '/')) != NULL) cp++;
      else                                     cp = thisDir;

    /* Only create the album if "mid" files exist! */
    if(Fp == NULL) {
      memset(Album, '\0', sizeof(Album));
      sprintf(Album, "%s/%s", ap, cp);

      printf("Creating Album %s\n", cp);

      if((Fp = fopen(Album, "w")) == NULL) {
	memset(Msg, '\0', sizeof(Msg));
	sprintf(Msg, "bldabmsProcDirProcList(%s) - %s", thisDir, Album);
	perror(Msg);

	return(-1);
      } else {
	chmod(Album, 0755);
	fprintf(Fp, "#!%s -a\n", bldabmsArgv0);
      }
    }
    fprintf(Fp, "%s\n", (char *)node);
  }

  if(Fp != NULL) fclose(Fp);

  return(rc);
}
/*---------------------------------------------------------------------------*/
